<?php

namespace Drupal\email_validate\Form;

use Drupal\Core\DrupalKernel;
use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\TypedData\TypedDataTrait;

/**
 * Configure Email extended validation settings for this site.
 */
class EmailValidateForm extends ConfigFormBase {

  use TypedDataTrait;

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'email_validate_settings';
  }

  /**
   * {@inheritdoc}
   */
  protected function getEditableConfigNames() {
    return ['email_validate.settings'];
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    $config = $this->config('email_validate.settings');
    $constraints = $this->getConstraintsAll();

    foreach ($constraints as $constraintKey) {
      $constraintClass = 'Drupal\email_validate\Plugin\Validation\Constraint\\' . $constraintKey;
      $constraint = new $constraintClass();

      $form[$constraintKey] = [
        '#title' => $constraint->title,
        '#type' => 'details',
        '#open' => !empty($config->get($constraintKey)),
        '#tree' => TRUE,
      ];
      if ($config->get($constraintKey)) {
        $form[$constraintKey]['summary'] = ['#markup' => '<small>' . $this->t('Enabled') . '</small>'];
      }

      $form[$constraintKey]['enable'] = [
        '#type' => 'checkbox',
        '#title' => $this->t('Enable "@name".', ['@name' => $constraint->title]),
        '#default_value' => $config->get($constraintKey) ?? '',
        '#description' => $constraint->description,
      ];

      // Add constraint settings fields.
      $form[$constraintKey] += $constraint->getSettingsForm($config);
    }

    // Collapsible details.
    $form['#attached']['library'][] = 'core/drupal.collapse';
    // $form['#attached']['library'][] = 'claro/node-form';
    // $form['#attached']['library'][] = 'claro/drupal.responsive-detail';
    // $form['#attached']['library'][] = 'claro/drupal.nav-tabs';
    return parent::buildForm($form, $form_state);
  }

  /**
   * Return array of additional constraints fields.
   *
   * @param string $constraint
   *   Constraint name.
   *
   * @return int[]|string[]
   *   Array of field keys.
   */
  private function getConstraintAddFields($constraint) {
    $constraintClass = 'Drupal\email_validate\Plugin\Validation\Constraint\\' . $constraint;
    $constraint = new $constraintClass();
    return array_keys($constraint->getSettingsForm());
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $constraints = $this->getConstraintsAll();
    $constraints_enabled = [];
    $values = $form_state->getValues();

    foreach ($constraints as $constraint) {
      $is_enable = $values[$constraint]['enable'];
      $this->config('email_validate.settings')
        ->set($constraint, $is_enable)
        ->save();

      // Update additional constraint fields.
      $addFields = $this->getConstraintAddFields($constraint);
      if ($addFields) {
        foreach ($addFields as $addField) {
          $this->config('email_validate.settings')
            ->set($addField, $values[$constraint][$addField] ?? '')
            ->save();

        }
      }
      // Save to enabled constraints list.
      if ($is_enable) {
        $constraints_enabled[] = $constraint;
      }
    }
    // Update config.
    $this->config('email_validate.settings')
      ->set('constraints_enabled', $constraints_enabled)
      ->save();

    parent::submitForm($form, $form_state);

    // Reset field base info cache.
    $this->resetCache();
  }

  /**
   * Update field base info to apply the changes.
   * See drupal_flush_all_caches().
   */
  private function resetCache() {
    $kernel = NULL;
    if (!$kernel instanceof DrupalKernel) {
      $kernel = \Drupal::service('kernel');
      $kernel->invalidateContainer();
      $kernel->rebuildContainer();
    }

    // Clear all plugin caches.
    \Drupal::service('plugin.cache_clearer')->clearCachedDefinitions();
  }

  /**
   * Return all available Constraints for email field.
   *
   * @return array|int[]|string[]
   *   Array of constraints plugins.
   */
  private function getConstraintsAll() {
    static $cache = [];

    if (!$cache) {
      /**
       * @var \Drupal\Core\File\FileSystemInterface $file_system
       */
      $file_system = \Drupal::service('file_system');
      $module_handler = \Drupal::service('module_handler');
      $module_path = $module_handler->getModule('email_validate')->getPath();
      $module_driver_path = DRUPAL_ROOT . '/' . $module_path . '/src/Plugin/Validation/Constraint';
      if (is_dir($module_driver_path)) {
        $mask = '/.*Constraint.php$/';
        $constraints = $file_system->scanDirectory(
          $module_driver_path, $mask, [
            'key' => 'name',
            'recurse' => FALSE,
          ]
        );
      }
      $cache = array_keys($constraints);
    }

    return $cache;
  }

}
